package nl.rubend.zwemmen

import android.app.Activity
import android.content.SharedPreferences
import android.os.Bundle
import android.view.View
import android.widget.ArrayAdapter
import kotlinx.coroutines.*
import nl.rubend.zwemmen.databinding.ActivityMainBinding
import nl.rubend.zwemmen.providers.last.LastProvider
import nl.rubend.zwemmen.providers.records.RecordProvider


class MainActivity : Activity() {
    val http = CustomHttp()
    val steps = mutableListOf<Int>()
    lateinit var binding: ActivityMainBinding
    lateinit var storage: SharedPreferences
    val providers=mapOf(
        "Last" to LastProvider(),
        "Records" to RecordProvider(),
    )
    val map: Map<String, Map<String,Any>> = providers.keys.associateWith { mapOf("Not yet loaded!" to "not yet loaded!") }
    val response: MutableMap<String, Map<String, Any>> = map.toMutableMap()
    override fun onBackPressed() {
        if(steps.size==0) return
        steps.removeLast()
        show()
    }

    fun setAdapter(items: List<String>) {
        binding.list.adapter=ArrayAdapter(this,android.R.layout.simple_list_item_1, items)
    }
    fun setAdapterMap(items: Map<String,Any>) {
        setAdapter(items.keys.toList())
        println(items.values.toList()[0])
        binding.list.setOnItemClickListener { _, _, position, _ ->
            steps.add(position)
            if(steps.size==1) {
                binding.list.isEnabled=false
                binding.progressBar.visibility= View.VISIBLE
                CoroutineScope(Dispatchers.Main + Job()).launch {
                    withContext(Dispatchers.IO) {
                        val item = response.keys.toList()[position]
                        response[item] = providers[item]!!.getResponse(http, storage)
                    }
                    show()
                    binding.progressBar.visibility= View.GONE
                    binding.list.isEnabled=true
                }
            } else show()
        }
    }
    fun show() {
        var items: Map<String, Any> = response.toMap()
        steps.forEach { step ->
            @Suppress("UNCHECKED_CAST") //Casting to List<String> and Map<String, Any>
            if (items.values.toList()[step] is List<*>) return setAdapter(items.values.toList()[step] as List<String>)
            else items = items.values.toList()[step] as Map<String, Any>
        }
        setAdapterMap(items)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)
        show()
        println(response)
        storage = applicationContext.getSharedPreferences("storage", MODE_PRIVATE)!!
        binding.offline.isChecked=storage.getBoolean("offline",false)
        binding.offline.setOnCheckedChangeListener { _, isChecked ->
            with (storage.edit()) {
                putBoolean("offline", isChecked)
                apply()
            }
        }
        binding.swimmer.setText(storage.getString("swimmer",""))
        binding.saveSwimmer.setOnClickListener {
            with (storage.edit()) {
                putString("swimmer", binding.swimmer.text.toString())
                apply()
            }
        }

    }
}