package nl.rubend.zwemmen.providers

import android.content.SharedPreferences
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import nl.rubend.zwemmen.CustomHttp
import nl.rubend.zwemmen.only200
import nl.rubend.zwemmen.text
import java.lang.reflect.Type
import java.text.DateFormat
import java.util.*


abstract class DataProvider {
    protected abstract fun getData(json: String): Map<String, Any>
    protected abstract val url: String
    fun getResponse(http: CustomHttp, storage: SharedPreferences): Map<String, Any> {
        val swimmer=storage.getString("swimmer","")
        val swimmerUrl="$url/$swimmer"
        return try {
            if(storage.getBoolean("offline", false)) throw Exception("forced offline!")
            val response: MutableMap<String, Any> = mutableMapOf("Last downloaded: "+ DateFormat.getDateTimeInstance().format(Date()) to listOf<String>())
            val httpResponse = http.get(swimmerUrl).only200().text()
            val data = getData(httpResponse)
            response.putAll(data)
            with(storage.edit()) {
                putString(swimmerUrl, Gson().toJson(response))
                apply()
            }
            response
        } catch (exception: Exception) {
            exception.printStackTrace()
            println(exception)
            val type: Type = object : TypeToken<Map<String, Any>>() {}.type
            Gson().fromJson(storage.getString(swimmerUrl, "{\"no data!\": []}"), type)
        }
    }
}